require "tres_bot/version"
require 'circular_list'

module TresBot


  class Robot
    attr_accessor :current_position
    attr_accessor :facing
    attr_accessor :is_on_the_table

    def initialize
      self.current_position = [0,0]
      self.facing = 'NORTH'
      self.is_on_the_table = false
    end

    def place(position)
      self.is_on_the_table  = true
      self.current_position = [position[0],position[1]]
      self.facing           = position[2].upcase
    end

    def right
      c = ::CircularList::List.new(['NORTH','EAST','SOUTH','WEST'])
      self.facing = c.fetch_next(c.index(self.facing))
    end

    def left
      c = ::CircularList::List.new(['NORTH','EAST','SOUTH','WEST'])
      self.facing = c.fetch_previous(c.index(self.facing))
    end

    def compass(dir)
      dir = dir.downcase
      axis = ['north','south'].include?(dir) ? 'y':'x'
      operator = ['north','east'].include?(dir) ? '+' : '-'
      [axis,operator]
    end

    def move
      calibration = self.compass(self.facing)
      axis = calibration[0]
      operator = calibration[1]
      if axis == 'x'
        unless (self.current_position[0] == 0 && operator == '-') || (self.current_position[0] == 4 && operator == '+')
          self.current_position[0]=self.current_position[0].to_i.send(operator,1)
        end
      else
        unless (self.current_position[1] == 0 && operator == '-') || (self.current_position[1] == 4 && operator == '+')
          self.current_position[1]=self.current_position[1].to_i.send(operator,1)
        end
      end
    end

    def report
      rep = "#{self.current_position.join(', ')}, #{self.facing}"
      puts rep
      rep
    end

  end

  def self.place_command_valid?(command)
    if command[1].nil?
      return false
    else
      command[1].downcase.gsub(/\s+/, "").match(/[0-4]+[,]+[0-9]+[,]+[a-z]{4,5}/) ? true : false
    end
  end

  def self.execute_command(robot,command)
    @robot = robot
    command = command.split(' ')

    if command[0] != nil
      if command[0].downcase == 'place'
        if place_command_valid?(command)
          full_position = command[1].split(',')
          @robot.place(full_position)
        else
          puts "Your place command is invalid. Please try again."
        end
      elsif ['move', 'report','left','right'].include? command[0].downcase
        @robot.is_on_the_table ? @robot.send(command[0].downcase) : (puts "Put me on the table!")
      end
    end
  end

  def self.start
    puts 'Press Ctrl-C to exit'
    @robot = Robot.new

    until false == true
      command = gets
      execute_command(@robot,command)
    end
  end
end
